'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _filterObjectProps = require('../../modules/filterObjectProps');

var _StyleSheet = require('../../modules/StyleSheet');

var _StyleSheet2 = _interopRequireDefault(_StyleSheet);

var _CoreComponent = require('../CoreComponent');

var _CoreComponent2 = _interopRequireDefault(_CoreComponent);

var _ImageStylePropTypes = require('./ImageStylePropTypes');

var _ImageStylePropTypes2 = _interopRequireDefault(_ImageStylePropTypes);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _View = require('../View');

var _View2 = _interopRequireDefault(_View);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* global window */

var STATUS_ERRORED = 'ERRORED';
var STATUS_LOADED = 'LOADED';
var STATUS_LOADING = 'LOADING';
var STATUS_PENDING = 'PENDING';
var STATUS_IDLE = 'IDLE';

var imageStyleKeys = Object.keys(_ImageStylePropTypes2.default);

var styles = _StyleSheet2.default.create({
  initial: {
    alignSelf: 'flex-start',
    backgroundColor: 'transparent',
    backgroundPosition: 'center',
    backgroundRepeat: 'no-repeat',
    backgroundSize: 'cover'
  },
  img: {
    borderWidth: 0,
    height: 'auto',
    maxHeight: '100%',
    maxWidth: '100%',
    opacity: 0
  },
  children: {
    bottom: 0,
    left: 0,
    position: 'absolute',
    right: 0,
    top: 0
  },
  resizeMode: {
    contain: {
      backgroundSize: 'contain'
    },
    cover: {
      backgroundSize: 'cover'
    },
    none: {
      backgroundSize: 'auto'
    },
    stretch: {
      backgroundSize: '100% 100%'
    }
  }
});

var Image = (function (_React$Component) {
  _inherits(Image, _React$Component);

  function Image(props, context) {
    _classCallCheck(this, Image);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(Image).call(this, props, context));

    var uri = props.source.uri;
    // state

    _this.state = { status: uri ? STATUS_PENDING : STATUS_IDLE };
    // autobinding
    _this._onError = _this._onError.bind(_this);
    _this._onLoad = _this._onLoad.bind(_this);
    return _this;
  }

  _createClass(Image, [{
    key: '_createImageLoader',
    value: function _createImageLoader() {
      var source = this.props.source;

      this._destroyImageLoader();
      this.image = new window.Image();
      this.image.onerror = this._onError;
      this.image.onload = this._onLoad;
      this.image.src = source.uri;
      this._onLoadStart();
    }
  }, {
    key: '_destroyImageLoader',
    value: function _destroyImageLoader() {
      if (this.image) {
        this.image.onerror = null;
        this.image.onload = null;
        this.image = null;
      }
    }
  }, {
    key: '_onError',
    value: function _onError(e) {
      var onError = this.props.onError;

      var event = { nativeEvent: e };

      this._destroyImageLoader();
      this.setState({ status: STATUS_ERRORED });
      this._onLoadEnd();
      if (onError) onError(event);
    }
  }, {
    key: '_onLoad',
    value: function _onLoad(e) {
      var onLoad = this.props.onLoad;

      var event = { nativeEvent: e };

      this._destroyImageLoader();
      this.setState({ status: STATUS_LOADED });
      if (onLoad) onLoad(event);
      this._onLoadEnd();
    }
  }, {
    key: '_onLoadEnd',
    value: function _onLoadEnd() {
      var onLoadEnd = this.props.onLoadEnd;

      if (onLoadEnd) onLoadEnd();
    }
  }, {
    key: '_onLoadStart',
    value: function _onLoadStart() {
      var onLoadStart = this.props.onLoadStart;

      this.setState({ status: STATUS_LOADING });
      if (onLoadStart) onLoadStart();
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      if (this.state.status === STATUS_PENDING) {
        this._createImageLoader();
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (this.state.status === STATUS_PENDING && !this.image) {
        this._createImageLoader();
      }
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      if (this.props.source.uri !== nextProps.source.uri) {
        this.setState({
          status: nextProps.source.uri ? STATUS_PENDING : STATUS_IDLE
        });
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      this._destroyImageLoader();
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props;
      var accessibilityLabel = _props.accessibilityLabel;
      var accessible = _props.accessible;
      var children = _props.children;
      var defaultSource = _props.defaultSource;
      var resizeMode = _props.resizeMode;
      var source = _props.source;
      var style = _props.style;
      var testID = _props.testID;

      var isLoaded = this.state.status === STATUS_LOADED;
      var defaultImage = defaultSource.uri || null;
      var displayImage = !isLoaded ? defaultImage : source.uri;
      var resolvedStyle = (0, _filterObjectProps.pickProps)(style, imageStyleKeys);
      var backgroundImage = displayImage ? 'url("' + displayImage + '")' : null;

      /**
       * Image is a non-stretching View. The image is displayed as a background
       * image to support `resizeMode`. The HTML image is hidden but used to
       * provide the correct responsive image dimensions, and to support the
       * image context menu. Child content is rendered into an element absolutely
       * positioned over the image.
       */
      return _react2.default.createElement(
        _View2.default,
        {
          _className: 'Image',
          accessibilityLabel: accessibilityLabel,
          accessibilityRole: 'img',
          accessible: accessible,
          style: _extends({}, styles.initial, resolvedStyle, backgroundImage && { backgroundImage: backgroundImage }, styles.resizeMode[resizeMode]),
          testID: testID
        },
        _react2.default.createElement('img', { src: displayImage, style: styles.img }),
        children ? _react2.default.createElement(_View2.default, { children: children, pointerEvents: 'box-none', style: styles.children }) : null
      );
    }
  }]);

  return Image;
})(_react2.default.Component);

Image.propTypes = {
  accessibilityLabel: _CoreComponent2.default.propTypes.accessibilityLabel,
  accessible: _CoreComponent2.default.propTypes.accessible,
  children: _react.PropTypes.any,
  defaultSource: _react.PropTypes.object,
  onError: _react.PropTypes.func,
  onLoad: _react.PropTypes.func,
  onLoadEnd: _react.PropTypes.func,
  onLoadStart: _react.PropTypes.func,
  resizeMode: _react.PropTypes.oneOf(['contain', 'cover', 'none', 'stretch']),
  source: _react.PropTypes.object,
  style: _react.PropTypes.shape(_ImageStylePropTypes2.default),
  testID: _CoreComponent2.default.propTypes.testID
};
Image.stylePropTypes = _ImageStylePropTypes2.default;
Image.defaultProps = {
  accessible: true,
  defaultSource: {},
  resizeMode: 'cover',
  source: {},
  style: styles.initial
};
exports.default = Image;