'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _predefs = require('./predefs');

var _expandStyle = require('./expandStyle');

var _expandStyle2 = _interopRequireDefault(_expandStyle);

var _getStyleObjects = require('./getStyleObjects');

var _getStyleObjects2 = _interopRequireDefault(_getStyleObjects);

var _prefixer = require('./prefixer');

var _prefixer2 = _interopRequireDefault(_prefixer);

var _Store = require('./Store');

var _Store2 = _interopRequireDefault(_Store);

var _StylePropTypes = require('../StylePropTypes');

var _StylePropTypes2 = _interopRequireDefault(_StylePropTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Initialize the store with pointer-event styles mapping to our custom pointer
 * event classes
 */
var initialState = { classNames: _predefs.predefinedClassNames };
var options = { obfuscateClassNames: process.env.NODE_ENV === 'production' };
var createStore = function createStore() {
  return new _Store2.default(initialState, options);
};
var store = createStore();
var isRendered = false;

/**
 * Destroy existing styles
 */
var _destroy = function _destroy() {
  store = createStore();
  isRendered = false;
};

/**
 * Render the styles as a CSS style sheet
 */
var _renderToString = function _renderToString() {
  var css = store.toString();
  isRendered = true;
  return _predefs.resetCSS + '\n' + _predefs.predefinedCSS + '\n' + css;
};

/**
 * Process all unique declarations
 */
var create = function create(styles) {
  var rules = (0, _getStyleObjects2.default)(styles);

  rules.forEach(function (rule) {
    var style = (0, _expandStyle2.default)(rule);

    Object.keys(style).forEach(function (property) {
      if (!_StylePropTypes2.default[property]) {
        console.error('ReactNativeWeb: the style property "' + property + '" is not supported');
      } else {
        var value = style[property];
        // add each declaration to the store
        store.set(property, value);
      }
    });
  });

  // update the style sheet in place
  if (isRendered) {
    var stylesheet = document.getElementById('react-stylesheet');
    if (stylesheet) {
      stylesheet.textContent = _renderToString();
    } else if (process.env.NODE_ENV !== 'production') {
      console.error('ReactNativeWeb: cannot find "react-stylesheet" element');
    }
  }

  return styles;
};

/**
 * Accepts React props and converts inline styles to single purpose classes
 * where possible.
 */
var resolve = function resolve(_ref) {
  var _ref$className = _ref.className;
  var className = _ref$className === undefined ? '' : _ref$className;
  var _ref$style = _ref.style;
  var style = _ref$style === undefined ? {} : _ref$style;

  var _className = undefined;
  var _style = {};
  var expandedStyle = (0, _expandStyle2.default)(style);

  var classList = [className];
  for (var prop in expandedStyle) {
    if (!_StylePropTypes2.default[prop]) {
      continue;
    }
    var styleClass = store.get(prop, expandedStyle[prop]);

    if (styleClass) {
      classList.push(styleClass);
    } else {
      _style[prop] = expandedStyle[prop];
    }
  }

  _className = classList.join(' ');
  _style = _prefixer2.default.prefix(_style);

  return { className: _className, style: _style };
};

exports.default = {
  _destroy: _destroy,
  _renderToString: _renderToString,
  create: create,
  resolve: resolve
};